"use strict";
exports.__esModule = true;
exports.createCompiler = void 0;
var sass = require("sass");
var path = require("path");
var fs = require("fs");
var less = require("less");
var Compiler = /** @class */ (function () {
    function Compiler(options) {
        if (options === void 0) { options = { ext: '.css', sourceMap: false, type: 'sass' }; }
        this.options = {
            ext: '.css',
            sourceMap: false,
            type: 'sass'
        };
        var ext = options.ext, sourceMap = options.sourceMap, type = options.type;
        this.options = {
            ext: ext || '.css',
            type: type || 'sass',
            sourceMap: sourceMap !== null && sourceMap !== void 0 ? sourceMap : false
        };
    }
    // 对编译的内容中导入的过滤,比如导入的是.wxss文件
    // 并将其原样的注入到目标文件头部中
    Compiler.prototype.compilerContentImportFilter = function (content) {
        var matches = content.match(/@[^;]+;/g);
        console.log(matches);
        var appendContent = '';
        if (matches) {
            for (var _i = 0, matches_1 = matches; _i < matches_1.length; _i++) {
                var match = matches_1[_i];
                if (match.indexOf('.wxss') > -1) {
                    content = content.replace(match, '');
                    appendContent += (match + '\n');
                }
            }
        }
        return {
            filterContent: content,
            appendContent: appendContent
        };
    };
    Compiler.prototype._compileForLess = function (filepath, outFilepath) {
        try {
            var _a = this.compilerContentImportFilter(fs.readFileSync(filepath).toString()), appendContent_1 = _a.appendContent, filterContent = _a.filterContent;
            less.render(filterContent, function (err, output) {
                if (err) {
                    console.log('compiling error');
                    console.log('\n');
                    return;
                }
                fs.writeFileSync(outFilepath, appendContent_1 + output.css);
                console.log('complied...');
            });
        }
        catch (error) {
            console.log('compiling error');
            console.log('\n');
        }
    };
    Compiler.prototype._compilerForSass = function (filepath, outFilepath) {
        var _a = this.compilerContentImportFilter(fs.readFileSync(filepath).toString()), appendContent = _a.appendContent, filterContent = _a.filterContent;
        fs.writeFileSync(filepath + '.temp', filterContent);
        try {
            var result = sass.renderSync({
                file: filepath + '.temp',
                outFile: outFilepath,
                sourceMap: this.options.sourceMap
            });
            fs.unlinkSync(filepath + '.temp');
            fs.writeFileSync(outFilepath, appendContent + result.css.toString());
            this.options.sourceMap && fs.writeFileSync(outFilepath + '.map', result.map.toString());
            console.log('complied...');
        }
        catch (error) {
            console.log('compiling error');
            console.log('\n');
        }
    };
    Compiler.prototype.compile = function (filepath) {
        var ext = path.extname(filepath) || '';
        var basename = path.basename(filepath, ext);
        var outputExt = this.options.ext;
        var outputFullFileName = "" + basename + outputExt;
        var outFilepath = path.join(path.dirname(filepath), outputFullFileName);
        console.log('\n');
        console.log('compling...');
        console.log(filepath);
        console.log(outFilepath);
        if (this.options.type === 'less') {
            this._compileForLess(filepath, outFilepath);
        }
        else {
            this._compilerForSass(filepath, outFilepath);
        }
        // try {
        //   const result = sass.renderSync({
        //     file: filepath,
        //     outFile: outFilepath,
        //     sourceMap: this.options.sourceMap
        //   })
        //   fs.writeFileSync(outFilepath, result.css.toString())
        //   this.options.sourceMap && fs.writeFileSync(outFilepath + '.map', result.map.toString())
        //   console.log('complied...')
        // } catch (error) {
        //   console.log('complied error...')
        //   console.log('\n')
        // }
    };
    return Compiler;
}());
function createCompiler(options) {
    var compiler = new Compiler(options);
    return compiler;
}
exports.createCompiler = createCompiler;
