"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.createFileUtil = void 0;
var path = require("path");
var fs = require("fs");
var rule_1 = require("./rule");
/**
 * 根据所传入的文件夹路径，可获取文件夹，文件，根据匹配规则过滤文件
 * 以绝对路径为准
 * 文件工具
 */
var FileUtil = /** @class */ (function () {
    function FileUtil(filepath) {
        this.filepath = '';
        this.subFiles = [];
        this._allFiles = []; // 原有的所有文件
        this.allFiles = []; // 过滤后的所以文件
        this.allDirs = [];
        this.includeRules = [];
        this.excludeRules = [];
        if (!filepath) {
            throw new Error('文件路径未传入');
        }
        if (!path.isAbsolute(filepath)) {
            filepath = path.join(__dirname, filepath);
        }
        this.filepath = filepath;
        this._init();
    }
    FileUtil.prototype._init = function () {
        var filepath = this.filepath;
        if (FileUtil.isFile(filepath)) {
            this.subFiles = [];
            this.allFiles = [filepath];
        }
        else {
            var _a = this._search(filepath), allFiles = _a[0], allDirs = _a[1];
            this.allFiles = allFiles;
            this._allFiles = allFiles;
            this.allDirs = allDirs;
        }
    };
    FileUtil.prototype._search = function (filepath) {
        var _this = this;
        var files = fs.readdirSync(filepath);
        var result = [
            [],
            []
        ];
        files.forEach(function (filename) {
            var absPath = path.join(filepath, filename.toString());
            if (FileUtil.isFile(absPath)) {
                result[0].push(absPath);
            }
            else {
                result[1].push(absPath);
                var tempResult = _this._search(absPath);
                result[0] = __spreadArrays(result[0], tempResult[0]);
                result[1] = __spreadArrays(result[1], tempResult[1]);
            }
        });
        return result;
    };
    // 判断是否文件
    FileUtil.isFile = function (filepath) {
        return fs.statSync(filepath).isFile();
    };
    FileUtil.prototype.getSubFiles = function () {
        return this.subFiles;
    };
    FileUtil.prototype.getAllFiles = function (cb) {
        typeof cb === 'function' && (cb(this.allFiles));
        return this;
    };
    FileUtil.prototype.getSubDirs = function () {
    };
    FileUtil.prototype.includeRule = function (rules) {
        rules = rule_1.formatRules(rules);
        this.includeRules = __spreadArrays(this.includeRules, rules);
        if (!this.includeRules || this.includeRules.length < 1) {
            return this;
        }
        this._filterFile();
        return this;
    };
    FileUtil.prototype.excludeRule = function (rules) {
        rules = rule_1.formatRules(rules);
        this.excludeRules = __spreadArrays(this.excludeRules, rules);
        if (!this.excludeRules || this.excludeRules.length < 1) {
            return this;
        }
        this._filterFile();
        return this;
    };
    FileUtil.prototype.getIncludeRules = function (cb) {
        typeof cb === 'function' && cb(this.includeRules);
        return this;
    };
    FileUtil.prototype.getExcludeRules = function (cb) {
        typeof cb === 'function' && cb(this.excludeRules);
        return this;
    };
    // 过滤文件
    // rule (rules: any): FileUtil {
    //   rules = formatRules(rules)
    //   console.log('添加规则 ', rules)
    //   this.rules = [...this.rules, ...rules]
    //   if (!this.rules || this.rules.length < 1) {
    //     return
    //   }
    //   this._filterFile()
    //   return this
    // }
    FileUtil.prototype._filterFile = function () {
        var _this = this;
        var includeRules = this.includeRules;
        var excludeRules = this.excludeRules;
        this.allFiles = this._allFiles.reduce(function (result, filepath) {
            var relativePath = path.relative(_this.filepath, filepath);
            if (!rule_1.isMatchRules(relativePath, excludeRules) && rule_1.isMatchRules(relativePath, includeRules)) {
                result.push(filepath);
            }
            return result;
        }, []);
    };
    FileUtil.prototype.getAllDirs = function (cb) {
        typeof cb === 'function' && (cb(this.allDirs));
        return this;
    };
    return FileUtil;
}());
function createFileUtil(filepath) {
    var watcher = new FileUtil(filepath);
    return watcher;
}
exports.createFileUtil = createFileUtil;
