"use strict";
exports.__esModule = true;
exports.createService = void 0;
var compile_1 = require("./compile");
var watch_1 = require("./watch");
var fileUtil_1 = require("./fileUtil");
var path = require("path");
var rule_1 = require("./rule");
/**
 *
 * @param options
 * @param options.openWatch
 * @param options.watch { filepath, recursive, eventType }
 * @param options.compile { ext, includes, excludes, sourceMap }
 *
 */
function createService(options) {
    var _a, _b, _c;
    if (typeof options === 'string') {
        var type = 'sass';
        if (options === 'less') {
            type = 'less';
        }
        options = {
            compile: {
                type: type
            }
        };
    }
    options = (options || {});
    var watch = options.watch || {};
    var compile = options.compile || {};
    var compileCfg = {
        ext: compile.ext || '.wxss',
        type: compile.type || 'sass',
        includes: compile.includes === undefined ? (compile.type === 'less' ? '.less' : '.scss') : compile.includes,
        excludes: compile.excludes === undefined ? { type: 'prefix', value: 'node_modules' } : compile.excludes,
        sourceMap: (_a = compile.sourceMap) !== null && _a !== void 0 ? _a : false
    };
    var watchCfg = {
        filepath: watch.filepath,
        recursive: (_b = watch.recursive) !== null && _b !== void 0 ? _b : true,
        eventType: watch.eventType || watch_1.EventType.CHANGE
    };
    if (watchCfg.filepath === undefined) {
        var filepath = path.resolve(__dirname);
        watchCfg.filepath = filepath;
        // 若果环境是依赖包的话则截取node_modules上一层目录
        if (filepath.indexOf('node_modules') > -1) {
            watchCfg.filepath = path.normalize(filepath.split('node_modules')[0]);
        }
    }
    var workPath = watchCfg.filepath;
    var openWatch = (_c = options.openWatch) !== null && _c !== void 0 ? _c : true;
    // console.log(compileCfg)
    // console.log(watchCfg)
    // console.log(workPath)
    // // 创建编译器
    var compiler = compile_1.createCompiler(compileCfg);
    // 创建文件，并过滤后获取全部文件进行编译
    fileUtil_1.createFileUtil(workPath)
        .excludeRule(compileCfg.excludes)
        .includeRule(compileCfg.includes)
        .getAllFiles(function (files) {
        console.log('\n要编译的文件============================');
        console.log(files);
        files.forEach(function (file) {
            compiler.compile(file);
        });
    });
    if (openWatch) {
        console.log('\n开启监听...');
        watch_1.createWatcher(watchCfg)
            .filter(function (evtType, filename, fullpath) {
            var excludes = rule_1.formatRules(compileCfg.excludes);
            var includes = rule_1.formatRules(compileCfg.includes);
            return !rule_1.isMatchRules(filename, excludes) && rule_1.isMatchRules(filename, includes);
        })
            .listner(function (evtType, filename, fullpath) {
            compiler.compile(fullpath);
        });
    }
}
exports.createService = createService;
