"use strict";
exports.__esModule = true;
exports.isMatchRules = exports.formatRules = exports.RuleType = void 0;
var path = require("path");
// 匹配规则
var RuleType;
(function (RuleType) {
    RuleType["SUFFIX"] = "suffix";
    RuleType["PREFIX"] = "prefix";
    RuleType["FILE"] = "file";
})(RuleType = exports.RuleType || (exports.RuleType = {}));
/**
 * 格式化规则,转换成Rule接口格式类型的
 * @param rule 规则
 */
var _formatRule = function (rule) {
    var result = null;
    if (typeof rule === 'string') {
        result = {
            type: RuleType.SUFFIX,
            value: rule
        };
    }
    else if (rule && rule.type && rule.value) {
        result = {
            type: rule.type,
            value: rule.value
        };
    }
    if (result) {
        if (result.type === 'file' || result.type === 'prefix') {
            // 若果时文件或者前缀类型，则需要对只能匹配的内容进行格式化，并除去开头的\符合
            result.value = path.normalize(result.value).replace(/^\\/, '');
        }
    }
    return result;
};
/**
 * 格式化规则，返回数组
 * @param rules 规则
 */
function formatRules(rules) {
    var result = [];
    if (Array.isArray(rules)) {
        rules.forEach(function (rule) {
            rule = _formatRule(rule);
            rule && (result.push(rule));
        });
    }
    else {
        var rule = _formatRule(rules);
        rule && (result.push(rule));
    }
    return result;
}
exports.formatRules = formatRules;
/**
 * isMatchRules 是否匹配规则
 * @param {*} filepath 文件相对路径
 * @param {*} rules // 规则
 */
exports.isMatchRules = function (filepath, rules) {
    var ext = path.extname(filepath);
    var basename = path.basename(filepath, ext);
    var fullname = "" + basename + ext;
    for (var i = 0, len = rules.length; i < len; i++) {
        var _a = rules[i], type = _a.type, value = _a.value;
        var flag = false;
        switch (type) {
            case 'suffix':
                flag = fullname.endsWith(value);
                break;
            case 'prefix':
                flag = filepath.indexOf(value) === 0;
                break;
            case 'file':
                flag = filepath === value;
        }
        if (flag)
            return true;
    }
    return false;
};
