"use strict";
exports.__esModule = true;
exports.createWatcher = exports.EventType = void 0;
var path = require("path");
var fs = require("fs");
var EventType;
(function (EventType) {
    EventType["CHANGE"] = "change";
    EventType["RENAME"] = "rename";
})(EventType = exports.EventType || (exports.EventType = {}));
var Watcher = /** @class */ (function () {
    function Watcher(options) {
        this.options = {
            filepath: path.resolve(__dirname),
            recursive: true,
            eventType: EventType.CHANGE
        };
        this._filterCbs = [];
        // this.options.recursive = options.recursive ?? true
        this.options = Object.assign({}, this.options, options);
        this.options.filepath = options.filepath || path.resolve(__dirname);
        console.log('监听文件目录：', this.options.filepath, '\n');
    }
    /**
     * 用于判断事件类型是否存在于所配置项的事件类型里
     * @param evtType 事件类型
     * @param evtTypes 配置的事件类型
     */
    Watcher.prototype._inEvtTypes = function (evtType, evtTypes) {
        var flag = false;
        if (typeof evtTypes === 'string') {
            flag = evtTypes === evtType;
        }
        else if (Array.isArray(evtType)) {
            flag = evtTypes.includes(evtType);
        }
        return flag;
    };
    /**
     * 过滤的当前事件类型，符合的就通行
     * 配置项的事件为空时，就允许所以通行
     * @param currentEvtType 当前所监听的事件类型
     */
    Watcher.prototype._filterEvtTypes = function (currentEvtType) {
        var evtTypes = this.options.eventType;
        if (!evtTypes)
            return true;
        return this._inEvtTypes(currentEvtType, evtTypes);
    };
    Watcher.prototype.filter = function (cb) {
        if (typeof cb === 'function') {
            this._filterCbs.push(cb);
        }
        return this;
    };
    /**
     * 过滤回调是否通行
     * @param evtType //事件类型
     * @param filename // 文件相对路径
     * @param fullpath // 文件全路径
     */
    Watcher.prototype._passfilterCbs = function (evtType, filename, fullpath) {
        var filterCbs = this._filterCbs;
        for (var _i = 0, filterCbs_1 = filterCbs; _i < filterCbs_1.length; _i++) {
            var cb = filterCbs_1[_i];
            if (!cb(evtType, filename, fullpath))
                return false;
        }
        return true;
    };
    Watcher.prototype._listnerCb = function () {
    };
    /**
     * 开启监听, 符合过滤条件的才会执行回调
     * @param cb 监听回调
     */
    Watcher.prototype.listner = function (cb) {
        var _this = this;
        fs.watch(this.options.filepath, {
            recursive: this.options.recursive
        }, function (evtType, filename) {
            var fullpath = path.join(_this.options.filepath, filename);
            _this._filterEvtTypes(evtType)
                && _this._passfilterCbs(evtType, filename, fullpath)
                && cb(evtType, filename, fullpath);
        });
        return this;
    };
    return Watcher;
}());
function createWatcher(options) {
    var watcher = new Watcher(options);
    return watcher;
}
exports.createWatcher = createWatcher;
