
import * as sass from 'sass'
import * as path from 'path'
import * as fs from 'fs'
import * as less from 'less'
export interface Options {
  ext: string,
  sourceMap: boolean,
  type: string
}
class Compiler {
  private options: Options = {
    ext: '.css',
    sourceMap: false,
    type: 'sass'
  }
  constructor (options: Options = { ext: '.css', sourceMap: false, type: 'sass' }) {
    const { ext, sourceMap, type } = options
    this.options = {
      ext: ext || '.css',
      type: type || 'sass',
      sourceMap: sourceMap ?? false
    }
  }
  
  // 对编译的内容中导入的过滤,比如导入的是.wxss文件
  // 并将其原样的注入到目标文件头部中
  compilerContentImportFilter (content: string): object {
    const matches = content.match(/@[^;]+;/g)
    console.log(matches)
    let appendContent = ''
    if (matches) {
      for (const match of matches) {
        if (match.indexOf('.wxss') > -1) {
          content = content.replace(match, '')
          appendContent +=(match + '\n')
        }
      }
    }

    return {
      filterContent: content,
      appendContent: appendContent
    }
  }

  private _compileForLess (filepath: string, outFilepath: string) {
    try {
      const { appendContent,  filterContent } = this.compilerContentImportFilter(fs.readFileSync(filepath).toString()) as { appendContent: string,  filterContent: string}
      less.render(filterContent, (err, output) => {
        if (err) {
          console.log('compiling error')
          console.log('\n')
          return
        }
        fs.writeFileSync(outFilepath, appendContent + output.css)
        console.log('complied...')
      })
    } catch (error) {
      console.log('compiling error')
      console.log('\n')
    }
  }

  private _compilerForSass (filepath: string, outFilepath: string) {
    const { appendContent,  filterContent } = this.compilerContentImportFilter(fs.readFileSync(filepath).toString()) as { appendContent: string,  filterContent: string}
    fs.writeFileSync(filepath+'.temp', filterContent)
    try {
      const result = sass.renderSync({
        file: filepath + '.temp',
        outFile: outFilepath,
        sourceMap: this.options.sourceMap
      })
      fs.unlinkSync(filepath+'.temp')
      fs.writeFileSync(outFilepath, appendContent + result.css.toString())
      this.options.sourceMap && fs.writeFileSync(outFilepath + '.map', result.map.toString())
      console.log('complied...')
    } catch (error) {
      console.log('compiling error')
      console.log('\n')
    }
    
  }

  compile (filepath: string) {
    const ext = path.extname(filepath) || ''
    const basename = path.basename(filepath, ext)
    const outputExt = this.options.ext
    const outputFullFileName = `${basename}${outputExt}`
    const outFilepath = path.join(path.dirname(filepath), outputFullFileName)
    console.log('\n')
    console.log('compling...')
    console.log(filepath)
    console.log(outFilepath)
    if (this.options.type === 'less') {
      this._compileForLess(filepath, outFilepath)
    } else {
      this._compilerForSass(filepath, outFilepath)
    }
    // try {
    //   const result = sass.renderSync({
    //     file: filepath,
    //     outFile: outFilepath,
    //     sourceMap: this.options.sourceMap
    //   })
    //   fs.writeFileSync(outFilepath, result.css.toString())
    //   this.options.sourceMap && fs.writeFileSync(outFilepath + '.map', result.map.toString())
    //   console.log('complied...')
    // } catch (error) {
    //   console.log('complied error...')
    //   console.log('\n')
    // }
    
  }
}

export function createCompiler (options: Options): Compiler {
  const compiler = new Compiler(options)
  return compiler
}