import * as path from 'path'
import * as fs from 'fs'
import { formatRules, isMatchRules, Rule } from './rule'

/**
 * 根据所传入的文件夹路径，可获取文件夹，文件，根据匹配规则过滤文件
 * 以绝对路径为准
 * 文件工具
 */
class FileUtil {
  private filepath: string = ''
  private subFiles: Array<string> = []
  private _allFiles: Array<string> = [] // 原有的所有文件
  private allFiles: Array<string> = [] // 过滤后的所以文件
  private allDirs: Array<string> = []
  private includeRules: Array<Rule> = []
  private excludeRules: Array<Rule> = []
  constructor (filepath: string) {
    if (!filepath) {
      throw new Error('文件路径未传入')
    }
    if (!path.isAbsolute(filepath)) {
      filepath = path.join(__dirname, filepath)
    }
    this.filepath = filepath
    this._init()
  }

  private _init () {
    const filepath = this.filepath
    if (FileUtil.isFile(filepath)) {
      this.subFiles = []
      this.allFiles = [filepath]
    } else {
      const [ allFiles, allDirs ] = this._search(filepath)
      this.allFiles = allFiles
      this._allFiles = allFiles
      this.allDirs = allDirs
    }
  }

  private _search (filepath) {
    const files = fs.readdirSync(filepath)
    let result = [
      [],
      []
    ]
    files.forEach(filename => {
      const absPath = path.join(filepath, filename.toString())
      if (FileUtil.isFile(absPath)) {
        result[0].push(absPath)
      } else {
        result[1].push(absPath)
        const tempResult = this._search(absPath)
        result[0] = [...result[0], ...tempResult[0]]
        result[1] = [...result[1], ...tempResult[1]]
      }
    })
    return result
  }

  // 判断是否文件
  static isFile(filepath) {
    return fs.statSync(filepath).isFile()
  }

  getSubFiles (): Array<string> {
    return this.subFiles
  }

  getAllFiles (cb: Function): FileUtil {
    typeof cb === 'function' && (cb(this.allFiles))
    return this
  }

  getSubDirs () {

  }

  includeRule (rules: any):FileUtil {
    rules = formatRules(rules)
    this.includeRules = [...this.includeRules, ...rules]
    if (!this.includeRules || this.includeRules.length < 1) {
      return this
    }
    this._filterFile()
    return this
  }

  excludeRule (rules: any): FileUtil {
    rules = formatRules(rules)
    this.excludeRules = [...this.excludeRules, ...rules]
    if (!this.excludeRules || this.excludeRules.length < 1) {
      return this
    }
    this._filterFile()
    return this
  }

  getIncludeRules (cb: Function): FileUtil {
    typeof cb === 'function' && cb(this.includeRules)
    return this
  }

  getExcludeRules (cb: Function): FileUtil {
    typeof cb === 'function' && cb(this.excludeRules)
    return this
  }

  // 过滤文件
  // rule (rules: any): FileUtil {
  //   rules = formatRules(rules)
  //   console.log('添加规则 ', rules)
  //   this.rules = [...this.rules, ...rules]
  //   if (!this.rules || this.rules.length < 1) {
  //     return
  //   }
  //   this._filterFile()
  //   return this
  // }

  private _filterFile () {
    const includeRules = this.includeRules
    const excludeRules = this.excludeRules
    this.allFiles = this._allFiles.reduce((result, filepath) => {
      const relativePath = path.relative(this.filepath, filepath)
      if (!isMatchRules(relativePath, excludeRules) && isMatchRules(relativePath, includeRules)) {
        result.push(filepath)
      }
      return result
    }, [])
  }

  getAllDirs (cb: Function): FileUtil {
    typeof cb === 'function' && (cb(this.allDirs))
    return this
  }
  
}


export function createFileUtil (
  filepath: string
): FileUtil {
  const watcher: FileUtil = new FileUtil(filepath)
  return watcher
}