
import {
  Options as CompileOptions,
  createCompiler
} from './compile'

import {
  Options as WatchOptions,
  createWatcher,
  EventType
} from './watch'

import {
  createFileUtil
} from './fileUtil'
import * as path from 'path'

import { formatRules, isMatchRules } from './rule'

// 编译配置
interface CompileConfig extends CompileOptions {
  excludes: any,
  includes: any
}


interface WatchConfig extends WatchOptions {
}

export interface ServiceConfig {
  openWatch: boolean,
  watch: WatchOptions,
  compile: CompileConfig
}

/**
 * 
 * @param options 
 * @param options.openWatch
 * @param options.watch { filepath, recursive, eventType }
 * @param options.compile { ext, includes, excludes, sourceMap }
 * 
 */
export function createService (options: ServiceConfig | string) {
  
  if (typeof options === 'string') {
    let type: string = 'sass'
    if (options === 'less') {
      type = 'less'
    }
    options = {
      compile: {
        type
      }
    } as ServiceConfig
  }
  
  options = (options || {}) as ServiceConfig
  const watch:WatchOptions = options.watch || {} as WatchOptions
  const compile: CompileConfig = options.compile || ({} as CompileConfig)

  const compileCfg: CompileConfig = {
    ext: compile.ext || '.wxss',
    type: compile.type || 'sass',
    includes: compile.includes === undefined ? (compile.type === 'less' ? '.less' : '.scss' ): compile.includes,
    excludes: compile.excludes === undefined ? { type: 'prefix', value: 'node_modules' } : compile.excludes,
    sourceMap: compile.sourceMap ?? false
  }

  const watchCfg: WatchConfig = {
    filepath: watch.filepath,
    recursive: watch.recursive ?? true,
    eventType: watch.eventType || EventType.CHANGE
  }

  if (watchCfg.filepath === undefined) {
    const filepath = path.resolve(__dirname)
    watchCfg.filepath = filepath
    // 若果环境是依赖包的话则截取node_modules上一层目录
    if (filepath.indexOf('node_modules') > -1) {
      watchCfg.filepath = path.normalize(filepath.split('node_modules')[0])
    }
  }
  const workPath = watchCfg.filepath
  const openWatch = options.openWatch ?? true
  // console.log(compileCfg)
  // console.log(watchCfg)
  // console.log(workPath)

  // // 创建编译器
  const compiler = createCompiler(compileCfg)

  // 创建文件，并过滤后获取全部文件进行编译
  createFileUtil(workPath)
    .excludeRule(compileCfg.excludes)
    .includeRule(compileCfg.includes)
    .getAllFiles((files) => {
      console.log('\n要编译的文件============================')
      console.log(files)
      files.forEach(file => {
        compiler.compile(file)
      })
    })
  
  if (openWatch) {
    console.log('\n开启监听...')
    createWatcher(watchCfg as WatchOptions)
      .filter((evtType, filename, fullpath) => {
        const excludes = formatRules(compileCfg.excludes)
        const includes = formatRules(compileCfg.includes)
        return !isMatchRules(filename, excludes) && isMatchRules(filename, includes)
      })
      .listner((evtType, filename, fullpath) => {
        compiler.compile(fullpath)
      })
  }
}