import * as path from 'path'
// 匹配规则
export enum RuleType {
  SUFFIX = 'suffix',
  PREFIX = 'prefix',
  FILE = 'file'
}

export interface Rule {
  type: RuleType,
  value: string
}

/**
 * 格式化规则,转换成Rule接口格式类型的
 * @param rule 规则
 */
const _formatRule = (rule: Rule | string): Rule => {
  let result = null
  if (typeof rule === 'string') {
    result = {
      type: RuleType.SUFFIX,
      value: rule
    }
  } else if (rule && rule.type && rule.value) {
    result = {
      type: rule.type,
      value: rule.value
    }
  }
  if (result) {
    if (result.type === 'file' || result.type === 'prefix') {
      // 若果时文件或者前缀类型，则需要对只能匹配的内容进行格式化，并除去开头的\符合
      result.value = path.normalize(result.value).replace(/^\\/, '')
    }
  }
  return result
}

/**
 * 格式化规则，返回数组
 * @param rules 规则
 */
export function formatRules (
  rules: Rule | Array<Rule> | string
): Array<Rule> {
  let result: Array<Rule> = []

  if (Array.isArray(rules)) {
    rules.forEach(rule => {
      rule = _formatRule(rule)
      rule && (result.push(rule))
    })
  } else {
    const rule = _formatRule(rules)
    rule && (result.push(rule))
  }
  return result
}

/**
 * isMatchRules 是否匹配规则
 * @param {*} filepath 文件相对路径
 * @param {*} rules // 规则
 */
export const isMatchRules = (filepath: string, rules: any): boolean => {
  const ext = path.extname(filepath)
  const basename = path.basename(filepath, ext)
  const fullname = `${basename}${ext}`
  for (let i = 0, len = rules.length; i < len; i++) {
    const { type, value } = rules[i]
    let flag = false
    switch (type) {
      case 'suffix': 
        flag = fullname.endsWith(value)
        break
      case 'prefix':
        flag = filepath.indexOf(value) === 0
        break
      case 'file':
        flag = filepath === value
    }
    if (flag) return true
  }
  return false
}