
import * as path from 'path'
import * as fs from 'fs'

/**
 * 配置项
 */
export interface Options {
  filepath: string, // 文件路径
  recursive: boolean, // 是否监听子孙目类
  eventType: EventType | Array<EventType> // 要监听的事件类型
}

export enum EventType {
  CHANGE = 'change',
  RENAME = 'rename'
}

class Watcher {
  options: Options = {
    filepath: path.resolve(__dirname),
    recursive: true,
    eventType: EventType.CHANGE
  }
  _filterCbs: Array<Function> = []
  constructor (options: Options) {
    // this.options.recursive = options.recursive ?? true
    this.options = Object.assign({}, this.options, options)
    this.options.filepath = options.filepath || path.resolve(__dirname)
    console.log('监听文件目录：', this.options.filepath, '\n')
  }

  /**
   * 用于判断事件类型是否存在于所配置项的事件类型里
   * @param evtType 事件类型
   * @param evtTypes 配置的事件类型
   */
  _inEvtTypes (evtType: EventType, evtTypes: EventType | Array<EventType>): boolean {
    let flag: boolean = false
    if (typeof evtTypes === 'string') {
      flag = evtTypes === evtType
    } else if (Array.isArray(evtType)) {
      flag = evtTypes.includes(evtType)
    }
    return flag
  }

  /**
   * 过滤的当前事件类型，符合的就通行
   * 配置项的事件为空时，就允许所以通行
   * @param currentEvtType 当前所监听的事件类型
   */
  _filterEvtTypes (currentEvtType: EventType): boolean {
    const evtTypes: EventType | Array<EventType> = this.options.eventType
    if (!evtTypes) return true
    return this._inEvtTypes(currentEvtType, evtTypes)
  }


  filter (cb: Function) {
    if (typeof cb === 'function') {
      this._filterCbs.push(cb)
    }
    return this
  }

  /**
   * 过滤回调是否通行
   * @param evtType //事件类型
   * @param filename // 文件相对路径
   * @param fullpath // 文件全路径
   */
  _passfilterCbs (evtType: EventType, filename: string, fullpath: string): boolean {
    const filterCbs = this._filterCbs
    for (const cb of filterCbs) {
      if (!cb(evtType, filename, fullpath)) return false
    }
    return true
  }

  _listnerCb () {

  }

  /**
   * 开启监听, 符合过滤条件的才会执行回调
   * @param cb 监听回调
   */
  listner (cb: Function): Watcher {
    fs.watch(
      this.options.filepath,
      {
        recursive: this.options.recursive
      },
      (evtType: string, filename: string) => {
        const fullpath = path.join(this.options.filepath, filename)
        this._filterEvtTypes(evtType as EventType)
        && this._passfilterCbs(evtType as EventType, filename, fullpath)
        && cb(evtType, filename, fullpath)
      }
    )
    return this
  }
}

export function createWatcher (
  options: Options
): Watcher {
  const watcher: Watcher = new Watcher(options)
  return watcher
}
